source("code/heredity.r")

unbiased.varest <- function(FF, Y, nfake){
  # Calculate unbiased estimate of the variance.
  #
  # INPUTS:
  # FF      Matrix containing the fake factor columns.
  # Y       Response vector.
  # nfake   Number of fake factors.
  #
  # OUTPUTS:
  # s.sq.f: unbiased estimate of the error variance.  
  # ---------------------------------------------------------------
  
  myQR <- qr(FF)                
  R <- qr.R(myQR)
  Q <- qr.Q(myQR)
  BB <- qr.solve(R,t(Q)%*%Y)              
  Yhat <- FF%*%BB
  s.sq.f <- (t(Yhat)%*%Y)/nfake
  return(s.sq.f)
}

fit.so <- function(x, X2, Y){
  # Fit a model including a subset of the second-order effects. 
  # For parallel computations with the 'apply' function.
  #
  # INPUTS:
  # x       Numeric vector indicating a subset of the second-order 
  #         effects.
  # X2      Matrix including the contrast vector associated to all
  #         second-order effects.
  # Y       Response vector.
  #
  # OUTPUTS:
  # rss:    Residual sum of squares of the sub-model.  
  # ---------------------------------------------------------------
  
  Xint <- cbind(1,X2[,x]) # include the intercept
  myQR <- qr(Xint)                
  tryCatch({
    R <- qr.R(myQR)
    Q <- qr.Q(myQR)
    BB <- qr.solve(R,t(Q)%*%Y)              
    Yhat <- Xint%*%BB 
    e <- Y - Yhat
    rss <- sum(e^2)
    return(rss)}, error = function(err){return(NA)})
}

TwoStage <- function(y, D, fake.factors, a = 0.05, b = 0.2, nSO = 0){
  # Two-stage approach of Jones, B. and Nachtsheim, C. J. (2017). 
  # "Effective design-based model selection for definitive screening 
  # designs." Technometrics, 59:319-329.
  #
  # INPUTS:
  # y             Response vector.
  # D             Definitive screening design including both real  
  #               and fake factors.
  # fake.factors  Numeric vector to indicate the fake factors in D.
  # a             Significance level for the t-tests for the linear 
  #               main effects.
  # b             Significance level for the F-tests for the 
  #               second-order efffects.
  # nSO           Number of active second-order effects.
  #
  # OUTPUTS:
  # activeTrue:   Binary vector indicating the effects declared 
  #               active. 1: effects is active; 0: effect not active.
  # ---------------------------------------------------------------
  
  Y <- y - mean(y) # Center responses.
  D <- as.matrix(D)
  noF <- ncol(D)
  nruns <- nrow(D)
  var.den <- 1/(sum(D[,1]^2))
  allfactors <- 1:noF
  factors <- setdiff(allfactors, fake.factors)
  m <- length(factors)
  
  # Model matrix including all linear main effects,  
  # all quadratic main effects and all two-factor interactions.
  getmats <- heredity(D[,factors])
  modmat <- getmats[[1]][,2:ncol(getmats[[1]])]
  neffects <- ncol(modmat)
  
  # Centered model matrix.
  X2 <- (diag(nruns) - matrix(1,ncol=nruns,nrow=nruns))%*%modmat 
  rel <- getmats[[2]][2:(m+1),-1]
  
  # 1. Regress yc on the matrix D.--------------------------------
  myQR <- qr(D)               
  R <- qr.R(myQR)
  Q <- qr.Q(myQR)
  Bhat <- qr.solve(R,t(Q)%*%Y)              
  yME <- D%*%Bhat              
  
  # 2. Response data for the second-order effects.----------------
  ySO <- Y - yME
  
  # 3. Extract the estimates of the first m factors.--------------
  Bd.hat <- Bhat[factors,]
  
  # 4. t-tests for the main effects.------------------------------
  Df <- D[,fake.factors]
  nfake <- length(fake.factors)
  s.sq.f <- unbiased.varest(Df,y,nfake) # Unbiased estimate of variance.
  var.mes <- s.sq.f*var.den
  t.ratio.me <- Bd.hat/sqrt(var.mes)[1]
  pvalues <- 2*pt(-abs(t.ratio.me), df = nfake)
  sig.fact <- pvalues < a
  activeMEs <- which(sig.fact)
  mME <- sum(sig.fact)
  
  if(mME == 1){
    activeTrue <- rep(0, neffects)
    activeTrue[c(activeMEs)] <- 1
    return(activeTrue)
  }
  if(mME == 0){
    activeTrue <- rep(0, neffects)
    return(activeTrue)
  }
  
  # Pool nonsignificant effects with the fake factors.
  nonsif.fact <- factors[which(!sig.fact)]
  new.fake <- c(fake.factors, nonsif.fact)
  mf <- length(new.fake)
  s.sq.f <- unbiased.varest(D[,c(new.fake)],y,mf)
  
  # 5. Identify the active second-order effects.----------------
  H <- rel[activeMEs,]
  # Select interactions following strong effect heredity.
  cand_so <- which(apply(as.matrix(H),2,sum)>1) 
  TSS <- t(ySO)%*%ySO
  F.stat <- (TSS/noF)/s.sq.f
  f.crit <- qf(b, df1 = noF, df2 = mf, lower.tail = F)
  
  # Guided subset selection.
  if(nSO == 0){
    max.so <- min(length(cand_so), noF/2)
    } else {max.so <- min(length(cand_so), nSO)}
  active2FIs <- NULL
  citer <- 1
  while(F.stat > f.crit && citer <= max.so){
    int.set <- combn(cand_so, citer)
    rss.int.mod <- apply(int.set, MARGIN = 2, FUN = fit.so, X2 = X2, Y = ySO)
    sel.int <- which.min(rss.int.mod)
    R.r <- min(rss.int.mod, na.rm = T)
    active2FIs <- int.set[,sel.int] 
    
    F.stat <- (R.r/(noF - citer))/s.sq.f
    f.crit <- qf(b, df1 = noF - citer, df2 = mf, lower.tail = F)
    citer <- citer + 1 
  }
  
  # Effects declared active. 
  activeTrue <- rep(0, neffects)
  activeTrue[c(activeMEs, active2FIs)] <- 1
  return(activeTrue)
}


