ProjStatTwo <- function(sset, sDSD){
  # Evaluate projections of a standard definitive screening design for parallel 
  # calculations with the function 'parApply'.
  #
  # INPUTS:
  # sset      Numeric vector containig the set of columns to drop from the design.
  # sDSD      Numeric matrix containing a standard definitive screening design (sDSD).
  # OUTPUTS:
  # result:   Vector indicating the columns drop from the design and the
  #           average absolute correlation, the maximum absolute correlation
  #           and the sum of suqared correlation between pairs of two-factor
  #           interaction (2FI) contrast vectors of the projected design.
  # -----------------------------------------------------------------------------
  
  # Create projected definitive screening design (pDSD).
  pDSD <- sDSD[, -1*sset] 
  m <- ncol(pDSD) # Number of factors in the pDSD.
  namesdrop <- colnames(sDSD[, sset]) # Names of dropped columns.
  
  
  library(Matrix)
  Corrpairs <- function(s, XX){
    # Compute absolute correlation between two columns in a matrix X.
    #
    # INPUTS:
    # s         Numeric vector indicating the two columns.
    # XX        Numeric matrix.
    #
    # OUTPUTS:
    # result:   Absolute correlation between the columns.
    # -----------------------------------------------------------------------------
    Jcharact <- cor(XX[, s[1]], XX[, s[2]])
    Jch <- abs(Jcharact)
    return(Jch)
  }
  
  # Model matrix including the linear main effect contrast vectors and the 2FI 
  # contrast vectors.
  M <- model.matrix(~.^2-1, pDSD) 
  p <- ncol(M) 

  TWOFI <- M[, (m+1):p] # 2FI contrast vectors.
  pchoose2 <- combn(p - m, 2)
  
  # Compute criteria.
  Corr <- apply(pchoose2, MARGIN = 2, FUN = Corrpairs, XX = TWOFI)
  maxbascorr <- max(Corr) 
  avcorr <- mean(Corr) 
  ssqcorr <- sum((Corr)^2) 
  
  # Report results.
  result <- c(paste(namesdrop, collapse = ' '), c(avcorr, maxbascorr, ssqcorr))
  return(result)
}
