source("code/TwoStep.r")

TwoStep.sims <- function(x, DSD, noF, ff = 7:10, aeffects, num_active_SO, nobs, SNR = 2){
  # Implementation of the simulation protocol for parallel calculations 
  # with the function 'parSapply'.
  #
  # INPUTS:
  # x               Number indicating the iteration.
  # DSD             Definitive screening design including both real  
  #                 and fake factors.
  # noF             Number of real factors.
  # ff              Numeric vector to indicate the position of the
  #                 fake factors in DSD.
  # aeffects        Numeric vector to indicate the position of 
  #                 the active effects in the model matrix
  #                 including all linear main effects, 
  #                 all quadratic main effects, and all two-factor 
  #                 interactions.
  # b               Significance level for the F-tests for the 
  #                 second-order efffects.
  # num_active_SO   Number of active second-order effects.
  # nobs            Number of observations in the design.
  # SNR             Singal-to-noise ratio for the active effects.
  #
  # OUTPUTS:
  # selected:       Binary vector indicating the effects declared 
  #                 active. 1: effects is active; 0: effect not active.
  # ---------------------------------------------------------------
  
  # Load auxiliary libraries.
  source("code/TwoStep.r")
  source("code/heredity.r")
  
  # Create model matrix and record number of active effects.
  D <- DSD[,1:noF]
  hered.mat <- heredity(D)
  X <- hered.mat[[1]][,-1] 
  num_active <- length(aeffects)
  
  # Simulate responses.
  acoeff <- rep(SNR, length(aeffects))*(((round(runif(num_active)))*2)-1)
  y <- X[,aeffects]%*%acoeff + rnorm(nobs)
  
  # Detect the active effects.
  selected <- TwoStage(y = y, D = DSD, fake.factors = ff, nSO = num_active_SO)
  return(selected)
}

active_inactive_effects <- function(DSD, noF, activeF, actQE = 1){
  # Determine active and inactive effects for the simulations.
  #
  # INPUTS:
  # DSD             Definitive screening design including both real  
  #                 and fake factors.
  # noF             Number of real factors.
  # activeF         Numeric vector to indicate the position of the
  #                 active factors DSD.
  # actQE           Number indicating the active quadratic main 
  #                 effect to be used. i: use the quadratic main 
  #                 effect of factor i as the active effect.  
  # num_active_SO   Number of active second-order effects.
  #
  # OUTPUTS:
  # result:        List with the following objects: 
  #                       - activeLEs     Indicaes indicating the active
  #                                       linear main effects.
  #                       - inacactiveLEs Indicaes indicating the inactive
  #                                       linear main effects.
  #                       - activeSOEs    Indicaes indicating the active
  #                                       second-order effects.
  #                       - inactiveSOEs  Indicaes indicating the inactive
  #                                       second-order effects.
  # ---------------------------------------------------------------
  
  source("code/heredity.r")
  
  # Calculate model and heredity matrices.
  D <- DSD[,1:noF]
  hered.mat <- heredity(D)
  X <- hered.mat[[1]][,-1] 
  relmat <- hered.mat[[2]][-1,-1] 
  
  H <- relmat[activeF,]
  cand_SO <- which(apply(H,2,sum)>1)
  cand_peq <- cand_SO[cand_SO <= 2*noF] 
  
  # Determine the active second-order effects following strong effect heredity.
  activeTWOFI <- cand_SO[cand_SO > 2*noF]
  activeQEs <- cand_peq[actQE] 
  activeSO <- c(activeQEs, activeTWOFI)
  
  # Determine inactive linear main effects and second-order effects.
  inactive_SO <- setdiff((noF+1):dim(X)[2],activeSO)
  inactive_LEs <- setdiff(1:noF,activeF)
  
  # Save results.
  result <- list()
  result[["activeLEs"]] <- activeF
  result[["inacactiveLEs"]] <- inactive_LEs
  result[["activeSOEs"]] <- activeSO
  result[["inactiveSOEs"]] <- inactive_SO
  return(result)
}

powers_typeI <- function(results, myeffects){
  # Calculate the range of powers for the active effects and
  # the type-I error rates for the inactive second-order effects.
  #
  # INPUTS:
  # results         p times iters binary matrix containig the active
  #                 effects determined in each simulation, p: number
  #                 of parameters; iters: number of iterations.
  # myeffects       List containing the indices for the
  #                 active and inactive effects. Output from the function
  #                 'active_inactive_effects'.
  #
  # OUTPUTS:
  # Powers_LEs      Range of powers for the linear main effects.
  # Powers_SOEs     Range of powers for the second-order effects. 
  # errors_LEs      Range of Type-I error rates for the linear 
  #                 main effects. 
  # errors_SOEs     Range of Type-I error rates for the second-order 
  #                 effects. 
  # ---------------------------------------------------------------
  
  average_results <- rowMeans(results)
  # Powers for active linear main effects.
  Powers_LEs <- range(average_results[myeffects$activeLEs])
  # Powers for active second-order effects.
  Powers_SOEs <- range(average_results[myeffects$activeSOEs])
  # Type-I error rates for inactive linear main effects.
  errors_LEs <- range(average_results[myeffects$inacactiveLEs])
  # Type-I error rates for inactive second-order effects.
  errors_SOEs <- range(average_results[myeffects$inactiveSOEs])
  
  return(c(Powers_LEs, Powers_SOEs, errors_LEs, errors_SOEs))
}


