## ========================SIMULATION PROTOCOL==================================
# This script shows how to carry out the simulations in Section 2 of 
# "Projections of Definitive Screening Designs by Dropping Columns: Selection
# and Evaluation" by Vazquez, A. R., Goos, P., and Schoen, E. D. 
## =============================================================================

# Load source files.
source("code/simulation_source.r")

# Read designs-------------------------------------------------------------------

# Design constructed by dropping the last columns from the 10-factor 21-run 
# definitive screening design (DSD) in Table 1 of the main text.
pDSDa <- read.csv("designs/pDSDa.csv", header = T) 
# Design constructed by dropping columsn C_6, C_8, C_9 and C_{10}.
pDSDb <- read.csv("designs/pDSDb.csv", header = T) 
# Number of factors under study.
noF <- 6
# Number of observations.
nobs <- nrow(pDSDa)
# Signal-to-noise ratio for the active effects.
SNR <- 2
# Number of iterations for the simulations.
iters <- 1000 


# Simulation Study-----------------------------------------------------------------
# Load auxiliary objects.
active_factors <- combn(1:noF, 4) 
ncases <- ncol(active_factors)
summary_results_a <-  summary_results_b <- matrix(NA, ncol = 9, nrow = ncases) 
labels_des <- c(rep('pDSDa', ncases), rep('pDSDb', ncases))

# Create a cluster to run the simulations in parallel.
library(snow)
cl <- makeSOCKcluster(c("localhost","localhost","localhost","localhost"))

for (i in 1:15){
  activeF <- active_factors[,i]
  
  # Identify active and inactive effects.
  myeffects <- active_inactive_effects(DSD=pDSDa, noF=noF, activeF = activeF)
  active_effects <- c(myeffects$activeLEs, myeffects$activeSOEs)
  nactSO <- length(myeffects$activeSOEs)
  
  # Run simulations.
  results_two_a <- parSapply(cl, 1:iters, FUN = TwoStep.sims, DSD = pDSDa, noF=noF, 
                             ff = 7:10, aeffects = active_effects, 
                             num_active_SO = nactSO, nobs = nobs, SNR = SNR)
  
  results_two_b <- parSapply(cl, 1:iters, FUN = TwoStep.sims, DSD = pDSDb, noF = noF, 
                             ff = 7:10, aeffects = active_effects, 
                             num_active_SO = nactSO, nobs = nobs, SNR = SNR)
  
  # Calculate ranges of powers and type-I error rates for the effects.
  range_power_errors_a <- powers_typeI(results_two_a, myeffects)
  range_power_errors_b <- powers_typeI(results_two_b, myeffects)
  summary_results_a[i,] <- c(i, range_power_errors_a)
  summary_results_b[i,] <- c(i, range_power_errors_b)
  
  print(i)
} 

# Save the results to a table.
all_results <- data.frame('Design' = labels_des, rbind(summary_results_a, summary_results_b))
colnames(all_results) <- c('Design', 'Assignment', 'minPowerLEs', 'maxPowerLEs',
                           'minPowerSOEs', 'maxPowerSOEs','minTypeIErrLEs', 'maxTypeIErrLEs',
                           'minTypeIErrSOEs', 'maxTypeIErrSOEs')
write.csv(all_results, file = "results/my_ranges_powers_errors.csv")
